#version 330

const int NUM_LIGHTS = 2;

in vec2 pass_textureCoords;
in vec3 pass_normals;
in vec3 pass_toLight[NUM_LIGHTS];
in vec3 pass_toCamera;

uniform sampler2D textureDiffuse;
uniform sampler2D textureSpecular;
uniform sampler2D textureNormal;
uniform float shininess;
uniform float shininessStrength;
uniform int transparency;
uniform vec3 lightColor[NUM_LIGHTS];
uniform vec3 attenuation[NUM_LIGHTS];

layout (location = 0) out vec4 out_Color;
layout (location = 1) out vec4 out_BrightColour;
layout (location = 2) out vec4 out_lightColor;

void main(void){
	
	if(transparency == 1 && texture(textureDiffuse, pass_textureCoords).a < 0.01){
		discard;
	}
	
	vec3 unitNormal = normalize(pass_normals);
	vec3 unitToCamera = normalize(pass_toCamera);
	
	vec3 totalDiffuse = vec3(0.0);
	vec3 totalSpecular = vec3(0.0);
	
	for(int i = 0; i < NUM_LIGHTS; i++){
		float toLightLength = length(pass_toLight[i]);
		float attenuationFactor = attenuation[i].x + (attenuation[i].y * toLightLength) + (attenuation[i].z * toLightLength * toLightLength);
		vec3 unitToLight = normalize(pass_toLight[i]);
		float nDotl = dot(unitNormal, unitToLight);
		nDotl = max(nDotl, 0.0);
		totalDiffuse += (nDotl * lightColor[i])/attenuationFactor;
		
		vec3 lightDirection = -unitToLight;
		vec3 reflectedLightDirection = reflect(lightDirection, unitNormal);
		float specularFactor = dot(reflectedLightDirection, unitToCamera);
		specularFactor = max(specularFactor, 0.0);
		float dampedFactor = pow(specularFactor, shininess);
		totalSpecular += (dampedFactor * shininessStrength * lightColor[i])/attenuationFactor;
	}
	
	totalDiffuse = max(totalDiffuse, 0.2);
	out_Color = vec4(totalDiffuse, 1.0) * texture(textureDiffuse, pass_textureCoords) + vec4(totalSpecular, 1.0) * texture(textureSpecular, pass_textureCoords);
	
	out_BrightColour = vec4(0, 0, 0, 1);
	if(out_Color.r * 0.2126 + out_Color.g * 0.7152 + out_Color.b * 0.0722 > 0.9){
		out_BrightColour = out_Color;
	}
	out_lightColor = vec4(0, 0, 0, 1);
}















