#version 330

const int NUM_LIGHTS = 2;

in vec3 position;
in vec2 textureCoords;
in vec3 normals;

uniform mat4 transformationMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;
uniform vec3 lightPosition[NUM_LIGHTS];

uniform vec4 clipPlane;

out vec2 pass_textureCoords;
out vec3 pass_normals;
out vec3 pass_toLight[NUM_LIGHTS];
out vec3 pass_toCamera;

void main(void){
	vec4 worldPosition = transformationMatrix * vec4(position, 1.0);
	gl_ClipDistance[0] = dot(worldPosition, clipPlane);
	
	gl_Position = projectionMatrix * viewMatrix * worldPosition;
	pass_textureCoords = textureCoords;
	pass_normals = (transformationMatrix * vec4(normals, 0.0)).xyz;
	for(int i = 0; i < NUM_LIGHTS; i++){
		pass_toLight[i] = lightPosition[i] - worldPosition.xyz;
	}
	pass_toCamera = (inverse(viewMatrix) * vec4(0, 0, 0, 1.0)).xyz - worldPosition.xyz;
}