#version 140

in vec2 textureCoords;

out vec4 out_Colour;

uniform sampler2D colorTexture;
uniform sampler2D depthTexture;
//uniform float focusDepth;
uniform float maxDeltaDepth;
uniform int maxKernelWidth;
const int targetWidth = 1280;

void main(void){
	
	float near = 0.1;
	float far = 1000.0;
	
	float focusDepth = texture(depthTexture, vec2(0.5, 0.5)).r;
	focusDepth = 2.0 * near * far / (far + near - (2.0 * focusDepth - 1.0) * (far - near));
	
	float depth = texture(depthTexture, textureCoords).r;
	depth = 2.0 * near * far / (far + near - (2.0 * depth - 1.0) * (far - near));
	
	float deltaDepth = abs(focusDepth - depth)/maxDeltaDepth;
	
	float kernelWidth = floor(deltaDepth * maxKernelWidth + 1);
	kernelWidth = clamp(kernelWidth, 0, maxKernelWidth);
	
	float pixelWidth = 1.0/targetWidth;
	float offset = kernelWidth/2.0 - 0.5;
	
	vec2 samplePos = textureCoords - vec2(offset*pixelWidth, 0);
	
	out_Colour = vec4(0, 0, 0, 0);
	
	for(int i = 0; i < kernelWidth; i++){
		out_Colour += texture(colorTexture, samplePos + i*vec2(pixelWidth, 0))/kernelWidth; 
	}
}