#version 330

const int NUM_LIGHTS = 2;

in vec2 textureCoords;
in vec3 out_normals;
in vec3 toLightVector[NUM_LIGHTS];

layout (location = 0) out vec4 out_Color;
layout (location = 1) out vec4 out_BrightColour;
layout (location = 2) out vec4 out_lightColor;

uniform sampler2D tex;

uniform vec3 lightColor[NUM_LIGHTS];
uniform vec3 lightAttenuation[NUM_LIGHTS];

void main(void) {
	
	vec3 total = vec3(0.0);
	
	for(int i = 0; i < NUM_LIGHTS; i++){
		float toLightLength = length(toLightVector[i]);
		float attenuationFactor = lightAttenuation[i].x + (lightAttenuation[i].y * toLightLength) + (lightAttenuation[i].z * toLightLength * toLightLength);
		vec3 unitToLight = normalize(toLightVector[i]);
		float nDotl = dot(out_normals, unitToLight);
		nDotl = max(nDotl, 0.0);
		total += (nDotl * lightColor[i])/attenuationFactor;
	}
	
	total = max(total, 0.25);
	out_Color = vec4(total, 1.0) * texture(tex, textureCoords);

	
	
	out_BrightColour = vec4(0, 0, 0, 1);
	out_lightColor = vec4(0, 0, 0, 1);
}