#version 140

in vec2 textureCoords;

out vec4 out_Colour;

uniform sampler2D colorTexture;
uniform sampler2D depthTexture;
//uniform float focusDepth;
uniform int targetHeight;
uniform float maxDeltaDepth;
uniform int maxKernelHeight;

void main(void){
	
	float near = 0.1;
	float far = 1000.0;
	
	float focusDepth = texture(depthTexture, vec2(0.5, 0.5)).r;
	focusDepth = 2.0 * near * far / (far + near - (2.0 * focusDepth - 1.0) * (far - near));
	
	float depth = texture(depthTexture, textureCoords).r;
	depth = 2.0 * near * far / (far + near - (2.0 * depth - 1.0) * (far - near));
	
	float deltaDepth = abs(focusDepth - depth)/maxDeltaDepth;
	
	float kernelHeight = floor(deltaDepth * maxKernelHeight + 1);
	kernelHeight = clamp(kernelHeight, 0, maxKernelHeight);
	
	float pixelHeight = 1.0/targetHeight;
	float offset = kernelHeight/2.0 - 0.5;
	
	vec2 samplePos = textureCoords - vec2(0, offset*pixelHeight);
	
	out_Colour = vec4(0, 0, 0, 0);
	
	for(int i = 0; i < kernelHeight; i++){
		out_Colour += texture(colorTexture, samplePos + i*vec2(0, pixelHeight))/kernelHeight; 
	}
}