#version 330

const int NUM_LIGHTS = 2;
const float waveStrength = 0.04;
const float shineDamper = 20.0;
const float reflectivity = 0.5;

in vec4 clipSpace;
in vec2 textureCoords;
in vec3 toCameraVector;
in vec3 fromLightVector[NUM_LIGHTS];

layout (location = 0) out vec4 out_Color;
layout (location = 1) out vec4 out_BrightColour;
layout (location = 2) out vec4 out_lightColor;

uniform sampler2D reflectionTexture;
uniform sampler2D refractionTexture;
uniform sampler2D dudvMap;
uniform sampler2D normalMap;
uniform sampler2D depthMap;
uniform vec3 lightColour[NUM_LIGHTS];

uniform float moveFactor;

void main(void) {

	vec4 waterColor = vec4(0, 0, 0.3, 1);

	vec2 ndc = (clipSpace.xy/clipSpace.w)/2.0 + 0.5;
	vec2 refractTexCoords = vec2(ndc.x, ndc.y);
	vec2 reflectTexCoords = vec2(ndc.x, -ndc.y);
	
	float near = 0.1;
	float far = 1000.0;
	
	float depth = texture(depthMap, refractTexCoords).r;
	float floorDistance = 2.0 * near * far / (far + near - (2.0 * depth - 1.0) * (far - near));
	
	depth = gl_FragCoord.z;
	float waterDistance = 2.0 * near * far / (far + near - (2.0 * depth - 1.0) * (far - near));	
	float waterDepth = floorDistance - waterDistance;	
	
	vec2 distortedTexCoords = texture(dudvMap, vec2(textureCoords.x + moveFactor, textureCoords.y)).rg*0.1;
	distortedTexCoords = textureCoords + vec2(distortedTexCoords.x, distortedTexCoords.y+moveFactor);
	vec2 totalDistortion = (texture(dudvMap, distortedTexCoords).rg * 2.0 - 1.0) * waveStrength * clamp(waterDepth/20.0, 0.0, 1.0);
	
	refractTexCoords += totalDistortion;
	refractTexCoords = clamp(refractTexCoords, 0.001, 0.999);
	
	reflectTexCoords += totalDistortion;
	reflectTexCoords.x = clamp(reflectTexCoords.x, 0.001, 0.999);
	reflectTexCoords.y = clamp(reflectTexCoords.y, -0.999, -0.001);
	
	vec4 reflectColour = texture(reflectionTexture, reflectTexCoords);
	vec4 refractColour = texture(refractionTexture, refractTexCoords);
	refractColour = mix(refractColour, waterColor, clamp(waterDepth/10.0, 0.0, 1.0));
	
	vec4 normalMapColour = texture(normalMap, distortedTexCoords);
	vec3 normal = vec3(normalMapColour.r * 2.0 - 1.0, normalMapColour.b*3.0, normalMapColour.g * 2.0 - 1.0);
	normal = normalize(normal);
	
	vec3 viewVector = normalize(toCameraVector);
	float refractiveFactor = dot(viewVector, normal);
	refractiveFactor = clamp(refractiveFactor, 0.0, 1.0);
	
	vec3 specularHighlights = vec3(0, 0, 0);
	for(int i = 0; i < NUM_LIGHTS; i++){
		vec3 reflectedLight = reflect(normalize(fromLightVector[i]), normal);
		float specular = max(dot(reflectedLight, viewVector), 0.0);
		specular = pow(specular, shineDamper);
		specularHighlights += lightColour[i] * specular * reflectivity * clamp(waterDepth/5.0, 0.0, 1.0);
	}
	out_Color = mix(reflectColour, refractColour, refractiveFactor);
	out_Color = out_Color + vec4(specularHighlights, 1.0);
	out_Color.a = clamp(waterDepth/1.0, 0.0, 1.0);
	
	
	out_BrightColour = vec4(0, 0, 0, 1);
	if(out_Color.r * 0.2126 + out_Color.g * 0.7152 + out_Color.b * 0.0722 > 0.9){
		out_BrightColour = out_Color;
	}
	
	out_lightColor = vec4(0, 0, 0, 1);
}